package dev.pfaff.glass;

import java.lang.foreign.ValueLayout;
import java.lang.invoke.MethodHandle;
import java.lang.invoke.MethodHandles;

import static java.lang.foreign.ValueLayout.ADDRESS;
import static java.lang.invoke.MethodHandles.explicitCastArguments;
import static java.lang.invoke.MethodHandles.identity;
import static java.lang.invoke.MethodType.methodType;

/**
 * Some native interop helpers.
 */
public final class Interop {
	private static final boolean IS_ARCH_64_BIT = ADDRESS.byteSize() == Long.BYTES;

	/**
	 * The value layout of an integer whose size is that of {@link ValueLayout#ADDRESS}.
	 */
	public static final ValueLayout ADDRESS_INT = IS_ARCH_64_BIT ? ValueLayout.JAVA_LONG : ValueLayout.JAVA_INT;

	/**
	 * The class of an integer whose size is that of {@link ValueLayout#ADDRESS}.
	 */
	public static final Class<?> C_ADDRESS_INT = IS_ARCH_64_BIT ? long.class : int.class;

	public static final MethodHandle MH_cast_signed_int_to_long = explicitCastArguments(identity(long.class),
			methodType(long.class, int.class));
	public static final MethodHandle MH_cast_signed_long_to_int = explicitCastArguments(identity(int.class),
			methodType(int.class, long.class));

	public static final MethodHandle MH_cast_unsigned_int_to_long;
	public static final MethodHandle MH_cast_unsigned_long_to_int = MH_cast_signed_long_to_int;

	static {
		var l = MethodHandles.lookup();
		try {
			MH_cast_unsigned_int_to_long = l.findStatic(Integer.class, "toUnsignedLong", methodType(long.class, int.class));
		} catch (NoSuchMethodException | IllegalAccessException e) {
			throw new RuntimeException(e);
		}
	}

	public static final MethodHandle MH_longIdentity = MethodHandles.identity(long.class);

	public static final MethodHandle MH_cast_usize_to_long = IS_ARCH_64_BIT
			? MH_longIdentity
			: MH_cast_unsigned_int_to_long;
	public static final MethodHandle MH_cast_isize_to_long = IS_ARCH_64_BIT
			? MH_longIdentity
			: MH_cast_signed_int_to_long;

	public static final MethodHandle MH_cast_long_to_usize = IS_ARCH_64_BIT
			? MH_longIdentity
			: MH_cast_unsigned_long_to_int;
	public static final MethodHandle MH_cast_long_to_isize = IS_ARCH_64_BIT
			? MH_longIdentity
			: MH_cast_signed_long_to_int;
}
