package dev.pfaff.glass;

import org.jspecify.annotations.NonNull;
import org.jspecify.annotations.Nullable;

/**
 * Basic operating system information.
 */
public sealed interface OS {
	public static final OS HOST = switch (System.getProperty("os.name")) {
		case null -> new Unknown(null);
		case "Mac OS X" -> new macOS();
		case "Linux" -> new Linux();
		case "Windows 10" -> new Windows._10();
		case "Windows 11" -> new Windows._11();
		case "Windows" -> new Windows.Unknown("Windows");
		case String name when name.startsWith("Windows ") -> new Windows.Unknown(name);
		case String name -> new Unknown(name);
	};

	public record macOS() implements OS {}
	public record Linux() implements OS {}

	public sealed interface Windows extends OS {
		public record _10() implements Windows {}
		public record _11() implements Windows {}
		public record Unknown(@NonNull String name) implements Windows {}
	}

	public record Unknown(@Nullable String name) implements OS {}
}
